#ifndef SYSTEM_MEMORY_H
#define SYSTEM_MEMORY_H

#include <assert.h>
#include <typeinfo>

//tells if a chunk of memory is valid (as best we can tell)
bool memValid(void *data);

//frees a hunk of memory allocated by memAlloc/memAllocNamed
void memFree(void *data);

//this way, if our data is truly invalid, this will complain where we're freeing it at
#define MEMFREE(s)	\
	{	assert(s);	\
		assert(memValid(s));	\
		memFree(s);	\
	}

//alloc a hunk of memory.  tag it in our system, to keep track of memory load and what not.
void *memAlloc(size_t size);
void *memAllocNamed(size_t size, const char *name);

//append filename, line, etc, to the string. someone show me how to append integer #defines to strings using preproc cmds
#define	MEMALLOC(s)	memAllocNamed(s, __FILE__)
#define	MEMALLOCNAMED(s,n)	memAllocNamed(s, \
	__FILE__ ":" n )
//	__FILE__##:##__LINE__##:## n )

//ease of access within your class =)
//why is there a distinction between new and new[] ?
#define CONTROL_MEMORY()	\
	static void *operator new(size_t size) { return MEMALLOCNAMED(size, "Memory Control"); }	\
	static void *operator new[](size_t size) { return MEMALLOCNAMED(size, "Memory Control"); }	\
	static void operator delete(void *ptr) { MEMFREE(ptr); }	\
	static void operator delete[](void *ptr) { MEMFREE(ptr); }

void memSystemInit();
void memSystemShutdown();

#endif	//SYSTEM_MEMORY_H